@@ -191,15 +191,17 @@ class Script:
 class PasswordManager:
 
     def __init__(self, prompt=None, password=None):
-        if prompt is None:
-            self._prompt = _ft.partial(_click.prompt, "Your remote sudo password", hide_input=True, err=True)
-        else:
+        if prompt is not None:
             self._prompt = prompt
         self._password = password if password else _os.environ.get("TUES_PW")
 
-    def get(self):
+    @staticmethod
+    def _prompt(message):
+        return _click.prompt(message, hide_input=True, err=True)
+
+    def get(self, message):
         if self._password is None:
-            self._password = self._prompt()
+            self._password = self._prompt(message)
 
         return self._password
 
@@ -513,7 +515,7 @@ class SudoWriter:
         if self.PROMPT_TOKEN in data:
             data = data.replace(self.PROMPT_TOKEN, b"")
             self._waiting = True
-            password = self._pm.get()
+            password = self._pm.get("Your remote sudo password")
             _log.debug("SudoWriter prompt reply on %r", self.sudo)
             self.sudo.write((password + "\n").encode())
             await self.sudo.drain()
@@ -629,6 +631,16 @@ async def _redirect_io(session, stdout, stderr, sudo):
     await session.redirect(stdin=None, stdout=stdout, stderr=stderr or stdout)
 
 
+class TuesClient(_ssh.SSHClient):
+
+    def __init__(self, pm):
+        super().__init__()
+        self._pm = pm
+
+    def password_auth_requested(self):
+        return self._pm.get("Login password")
+
+
 async def _run(run, pm, stdout=None, stderr=None): # pylint: disable=too-many-locals,too-many-branches
     """ The real "runner" function issued for every host """
     env = []
@@ -641,7 +653,13 @@ async def _run(run, pm, stdout=None, stderr=None): # pylint: disable=too-many-lo
     if run.connection is None:
         try:
             async with _timeout.timeout(10):
-                conn = await _ssh.connect(run.host, known_hosts=None, username=run.login_user, port=run.port or ())
+                conn = await _ssh.connect(
+                    run.host,
+                    known_hosts=None,
+                    username=run.login_user,
+                    port=run.port or (),
+                    client_factory=_ft.partial(TuesClient, pm),
+                )
         except Exception as e:
             raise TuesError(f"Could not connect to {run.host}: {e!r}") from e
     else:
